@@ -1,21 +1,8 @@
-/***********************************************************************
- *
- * Copyright (C) 2008, 2010, 2011, 2012 Graeme Gott <graeme@gottcode.org>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- ***********************************************************************/
+/*
+	SPDX-FileCopyrightText: 2008 Graeme Gott <graeme@gottcode.org>
+
+	SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "overview.h"
 
@@ -23,17 +10,19 @@
 
 #include <QGraphicsPixmapItem>
 #include <QGraphicsScene>
+#include <QIcon>
 #include <QSettings>
 #include <QWheelEvent>
 
+#include <algorithm>
 #include <cmath>
 
 //-----------------------------------------------------------------------------
 
 Overview::Overview(QWidget* parent)
-	: QGraphicsView(parent),
-	m_min_scale_level(0),
-	m_scale_level(0)
+	: QGraphicsView(parent)
+	, m_min_scale_level(0)
+	, m_scale_level(0)
 {
 	setWindowTitle(tr("Overview"));
 	setWindowFlags(Qt::Tool);
@@ -53,7 +42,7 @@ Overview::Overview(QWidget* parent)
 	reset();
 
 	// Restore geometry
-	QSettings settings;
+	const QSettings settings;
 	if (settings.contains("Overview/Geometry")) {
 		restoreGeometry(settings.value("Overview/Geometry").toByteArray());
 		setMinimumSize(size());
@@ -66,15 +55,15 @@ Overview::Overview(QWidget* parent)
 
 //-----------------------------------------------------------------------------
 
-void Overview::load(const QImage& image)
+void Overview::load(const QImage& image, qreal pixelratio)
 {
 	// Find minimum scale
-	m_min_scale_level = 9;
-	int side_max = qMax(image.width(), image.height()) * 0.9;
+	m_min_scale_level = ZoomSlider::maxScaleLevel();
+	const int side_max = std::max(image.width(), image.height()) * 0.9;
 	int side = 400;
 	if (side_max > side) {
-		for (int i = 9; i >= 0; --i) {
-			int side_test = std::floor(400.0 / ZoomSlider::scaleFactor(i));
+		for (int i = ZoomSlider::maxScaleLevel(); i >= 0; --i) {
+			const int side_test = std::floor(400.0 / ZoomSlider::scaleFactor(i));
 			if (side_test > side_max) {
 				break;
 			}
@@ -84,13 +73,13 @@ void Overview::load(const QImage& image)
 	} else {
 		side = side_max;
 	}
-	QPixmap pixmap = QPixmap::fromImage(image.scaled(side, side, Qt::KeepAspectRatio, Qt::SmoothTransformation), Qt::AutoColor | Qt::AvoidDither);
+	QPixmap pixmap = QPixmap::fromImage(image.scaled(side * pixelratio, side * pixelratio, Qt::KeepAspectRatio, Qt::SmoothTransformation), Qt::AutoColor | Qt::AvoidDither);
+	pixmap.setDevicePixelRatio(pixelratio);
 	zoom(m_min_scale_level);
 
 	// Resize window
-	bool default_size = m_default;
-	QSize size = transform().mapRect(pixmap.rect()).size();
-	setMinimumSize(size);
+	const bool default_size = m_default;
+	setMinimumSize(transform().mapRect(QRect(QPoint(0,0), pixmap.size() / pixelratio)).size());
 	if (default_size) {
 		resize(minimumSize());
 	}
@@ -104,11 +93,11 @@ void Overview::load(const QImage& image)
 void Overview::reset()
 {
 	// Prevent zooming
-	m_min_scale_level = 9;
+	m_min_scale_level = ZoomSlider::maxScaleLevel();
 	zoom(m_min_scale_level);
 
 	// Show loading icon
-	setPixmap(QPixmap(":/loading.png"));
+	setPixmap(QIcon::fromTheme("image-loading").pixmap(128,128));
 }
 
 //-----------------------------------------------------------------------------
@@ -123,7 +112,7 @@ void Overview::moveEvent(QMoveEvent* event)
 
 void Overview::hideEvent(QHideEvent* event)
 {
-	emit toggled(false);
+	Q_EMIT toggled(false);
 	QGraphicsView::hideEvent(event);
 }
 
@@ -142,7 +131,7 @@ void Overview::resizeEvent(QResizeEvent* event)
 
 void Overview::showEvent(QShowEvent* event)
 {
-	emit toggled(true);
+	Q_EMIT toggled(true);
 	QGraphicsView::showEvent(event);
 }
 
@@ -150,7 +139,7 @@ void Overview::showEvent(QShowEvent* event)
 
 void Overview::wheelEvent(QWheelEvent* event)
 {
-	if (event->delta() > 0) {
+	if (event->angleDelta().y() > 0) {
 		zoomIn();
 	} else {
 		zoomOut();
@@ -185,12 +174,12 @@ void Overview::zoomOut()
 
 void Overview::zoom(int level)
 {
-	m_scale_level = qBound(m_min_scale_level, level, 9);
-	float s = ZoomSlider::scaleFactor(m_scale_level);
+	m_scale_level = std::clamp(level, m_min_scale_level, ZoomSlider::maxScaleLevel());
+	const float s = ZoomSlider::scaleFactor(m_scale_level);
 	resetTransform();
 	scale(s, s);
 
-	Qt::ScrollBarPolicy policy = (m_scale_level > m_min_scale_level) ? Qt::ScrollBarAsNeeded : Qt::ScrollBarAlwaysOff;
+	const Qt::ScrollBarPolicy policy = (m_scale_level > m_min_scale_level) ? Qt::ScrollBarAsNeeded : Qt::ScrollBarAlwaysOff;
 	setHorizontalScrollBarPolicy(policy);
 	setVerticalScrollBarPolicy(policy);
 }
