@@ -1,21 +1,8 @@
-/***********************************************************************
- *
- * Copyright (C) 2008, 2010, 2011 Graeme Gott <graeme@gottcode.org>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- ***********************************************************************/
+/*
+	SPDX-FileCopyrightText: 2008 Graeme Gott <graeme@gottcode.org>
+
+	SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "window.h"
 
@@ -31,7 +18,6 @@
 #include <QCloseEvent>
 #include <QDialog>
 #include <QDialogButtonBox>
-#include <QFileInfo>
 #include <QGridLayout>
 #include <QLabel>
 #include <QMenu>
@@ -45,67 +31,77 @@
 //-----------------------------------------------------------------------------
 
 Window::Window(const QStringList& files)
-	: m_board(0)
+	: m_board(nullptr)
 {
-	setWindowTitle(tr("Tetzle"));
 	setAcceptDrops(true);
 	resize(640, 480);
 
 	// Add statusbar
+	m_completed = new QProgressBar(this);
+	m_completed->setFormat(tr("%p% complete"));
+	m_completed->setRange(0, 100);
+	m_completed->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
+	statusBar()->addPermanentWidget(m_completed);
+
 	m_slider = new ZoomSlider(this);
 	statusBar()->addPermanentWidget(m_slider);
 
-	m_completed = new QProgressBar(this);
-	m_completed->setRange(0, 100);
-	m_completed->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Maximum);
-	statusBar()->addPermanentWidget(m_completed);
-
 	statusBar()->setMinimumHeight(statusBar()->sizeHint().height());
 	m_slider->hide();
 	m_completed->hide();
 
 	// Add contents
 	m_board = new Board(this);
-	connect(m_board, SIGNAL(completionChanged(int)), m_completed, SLOT(setValue(int)));
-	connect(m_board, SIGNAL(finished()), this, SLOT(gameFinished()));
-	connect(m_board, SIGNAL(clearMessage()), statusBar(), SLOT(clearMessage()));
-	connect(m_board, SIGNAL(showMessage(const QString&)), statusBar(), SLOT(showMessage(const QString&)));
-	connect(m_board, SIGNAL(zoomChanged(int, float)), m_slider, SLOT(setValue(int, float)));
-	connect(m_slider, SIGNAL(valueChanged(int)), m_board, SLOT(zoom(int)));
+	connect(m_board, &Board::completionChanged, m_completed, &QProgressBar::setValue);
+	connect(m_board, &Board::finished, this, &Window::gameFinished);
+	connect(m_board, &Board::clearMessage, statusBar(), &QStatusBar::clearMessage);
+	connect(m_board, &Board::showMessage, statusBar(), &QStatusBar::showMessage);
+	connect(m_board, &Board::zoomChanged, m_slider, &ZoomSlider::setValue);
+	connect(m_slider, &ZoomSlider::valueChanged, this, [this](int value) {
+		m_board->zoom(value, true);
+	});
+	connect(m_slider, &ZoomSlider::zoomFit, m_board, &Board::zoomFit);
+	connect(m_slider, &ZoomSlider::zoomIn, m_board, &Board::zoomIn);
+	connect(m_slider, &ZoomSlider::zoomOut, m_board, &Board::zoomOut);
 	setCentralWidget(m_board);
 
 	// Add menus
 	QMenu* menu;
 
 	menu = menuBar()->addMenu(tr("&Game"));
-	QAction* choose_action = menu->addAction(tr("&Choose..."), this, SLOT(chooseGame()));
-	choose_action->setShortcuts(QList<QKeySequence>() << QKeySequence::New << QKeySequence::Open);
+	QAction* choose_action = menu->addAction(tr("&Choose..."), this, [this]() { chooseGame(QStringList()); });
+	choose_action->setShortcuts({ QKeySequence::New, QKeySequence::Open});
 	menu->addSeparator();
-	QAction* retrieve_pieces_action = menu->addAction(tr("&Retrieve Pieces"), m_board, SLOT(retrievePieces()), tr("Ctrl+R"));
+	QAction* retrieve_pieces_action = menu->addAction(tr("&Retrieve Pieces"), m_board, &Board::retrievePieces);
+	retrieve_pieces_action->setShortcut(tr("Ctrl+R"));
 	retrieve_pieces_action->setEnabled(false);
-	connect(m_board, SIGNAL(retrievePiecesAvailable(bool)), retrieve_pieces_action, SLOT(setEnabled(bool)));
+	connect(m_board, &Board::retrievePiecesAvailable, retrieve_pieces_action, &QAction::setEnabled);
 	menu->addSeparator();
-	QAction* quit_action = menu->addAction(tr("&Quit"), this, SLOT(close()), QKeySequence::Quit);
+	QAction* quit_action = menu->addAction(tr("&Quit"), this, &Window::close);
+	quit_action->setShortcut(QKeySequence::Quit);
 	quit_action->setMenuRole(QAction::QuitRole);
 
 	menu = menuBar()->addMenu(tr("&View"));
-	QAction* zoom_in_action = menu->addAction(tr("Zoom &In"), m_board, SLOT(zoomIn()), tr("+"));
+	QAction* zoom_in_action = menu->addAction(tr("Zoom &In"), m_board, &Board::zoomIn);
+	zoom_in_action->setShortcut(tr("+"));
 	zoom_in_action->setEnabled(false);
-	connect(m_board, SIGNAL(zoomInAvailable(bool)), zoom_in_action, SLOT(setEnabled(bool)));
-	QAction* zoom_out_action = menu->addAction(tr("Zoom &Out"), m_board, SLOT(zoomOut()), tr("-"));
+	connect(m_slider, &ZoomSlider::zoomInAvailable, zoom_in_action, &QAction::setEnabled);
+	QAction* zoom_out_action = menu->addAction(tr("Zoom &Out"), m_board, &Board::zoomOut);
+	zoom_out_action->setShortcut(tr("-"));
 	zoom_out_action->setEnabled(false);
-	connect(m_board, SIGNAL(zoomOutAvailable(bool)), zoom_out_action, SLOT(setEnabled(bool)));
-	m_zoom_fit_action = menu->addAction(tr("Best &Fit"), m_board, SLOT(zoomFit()));
+	connect(m_slider, &ZoomSlider::zoomOutAvailable, zoom_out_action, &QAction::setEnabled);
+	m_zoom_fit_action = menu->addAction(tr("Best &Fit"), m_board, &Board::zoomFit);
 	m_zoom_fit_action->setEnabled(false);
 	menu->addSeparator();
-	m_toggle_overview_action = menu->addAction(tr("Show O&verview"), m_board, SLOT(toggleOverview()), tr("Tab"));
+	m_toggle_overview_action = menu->addAction(tr("Show O&verview"), m_board, &Board::toggleOverview);
+	m_toggle_overview_action->setShortcut(tr("Tab"));
 	m_toggle_overview_action->setCheckable(true);
 	m_toggle_overview_action->setEnabled(false);
-	connect(m_board, SIGNAL(overviewToggled(bool)), m_toggle_overview_action, SLOT(setChecked(bool)));
+	connect(m_board, &Board::overviewToggled, m_toggle_overview_action, &QAction::setChecked);
 	menu->addSeparator();
 
 	QAction* fullscreen_action = menu->addAction(tr("F&ullscreen"));
-	connect(fullscreen_action, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
+	connect(fullscreen_action, &QAction::toggled, this, &Window::setFullScreen);
 	fullscreen_action->setCheckable(true);
 #if !defined(Q_OS_MAC)
 	fullscreen_action->setShortcut(tr("F11"));
@@ -114,15 +110,16 @@ Window::Window(const QStringList& files)
 #endif
 
 	menu = menuBar()->addMenu(tr("&Settings"));
-	menu->addAction(tr("&Appearance..."), this, SLOT(showAppearance()));
-	menu->addAction(tr("&Language..."), this, SLOT(setLocale()));
+	menu->addAction(tr("&Appearance..."), this, &Window::showAppearance);
+	menu->addAction(tr("&Language..."), this, &Window::setLocale);
 
 	menu = menuBar()->addMenu(tr("&Help"));
-	menu->addAction(tr("&Controls"), this, SLOT(showControls()), QKeySequence::HelpContents);
+	QAction* help_action = menu->addAction(tr("&Controls"), this, &Window::showControls);
+	help_action->setShortcut(QKeySequence::HelpContents);
 	menu->addSeparator();
-	QAction* about_action = menu->addAction(tr("&About"), this, SLOT(showAbout()));
+	QAction* about_action = menu->addAction(tr("&About"), this, &Window::showAbout);
 	about_action->setMenuRole(QAction::AboutRole);
-	QAction* about_qt_action = menu->addAction(tr("About &Qt"), qApp, SLOT(aboutQt()));
+	QAction* about_qt_action = menu->addAction(tr("About &Qt"), qApp, &QApplication::aboutQt);
 	about_qt_action->setMenuRole(QAction::AboutQtRole);
 
 	// Restore geometry
@@ -140,7 +137,7 @@ Window::Window(const QStringList& files)
 
 	// Create auto-save timer
 	QTimer* timer = new QTimer(this);
-	connect(timer, SIGNAL(timeout()), m_board, SLOT(saveGame()));
+	connect(timer, &QTimer::timeout, m_board, &Board::saveGame);
 	timer->start(300000);
 }
 
@@ -153,16 +150,6 @@ void Window::addImages(const QStringList& files)
 
 //-----------------------------------------------------------------------------
 
-void Window::changeEvent(QEvent* event)
-{
-	if (event->type() == QEvent::WindowStateChange) {
-		m_board->updateGL();
-	}
-	return QMainWindow::changeEvent(event);
-}
-
-//-----------------------------------------------------------------------------
-
 void Window::closeEvent(QCloseEvent* event)
 {
 	QSettings().setValue("Geometry", saveGeometry());
@@ -183,7 +170,7 @@ void Window::dragEnterEvent(QDragEnterEvent* event)
 
 void Window::dropEvent(QDropEvent* event)
 {
-	QStringList files = AddImage::dropEvent(event);
+	const QStringList files = AddImage::dropEvent(event);
 	if (!files.isEmpty()) {
 		chooseGame(files);
 	}
@@ -196,10 +183,8 @@ void Window::chooseGame(const QStringList& files)
 	m_board->saveGame();
 
 	ChooseGameDialog dialog(files, m_board->id(), this);
-	connect(&dialog, SIGNAL(accepted()), m_slider, SLOT(hide()));
-	connect(&dialog, SIGNAL(accepted()), m_completed, SLOT(hide()));
-	connect(&dialog, SIGNAL(newGame(const QString&, int)), m_board, SLOT(newGame(const QString&, int)));
-	connect(&dialog, SIGNAL(openGame(int)), m_board, SLOT(openGame(int)));
+	connect(&dialog, &ChooseGameDialog::newGame, m_board, &Board::newGame);
+	connect(&dialog, &ChooseGameDialog::openGame, m_board, &Board::openGame);
 	if (dialog.exec() == QDialog::Accepted) {
 		m_toggle_overview_action->setEnabled(true);
 		m_zoom_fit_action->setEnabled(true);
@@ -252,7 +237,7 @@ void Window::showControls()
 	dialog.setWindowTitle(tr("Controls"));
 
 	QGridLayout* layout = new QGridLayout(&dialog);
-	layout->setMargin(12);
+	layout->setContentsMargins(12, 12, 12, 12);
 	layout->setSpacing(0);
 	layout->setColumnMinimumWidth(1, 6);
 	layout->setRowMinimumHeight(7, 12);
@@ -272,7 +257,7 @@ void Window::showControls()
 	layout->addWidget(new QLabel(tr("Move mouse or W,A,D,S"), &dialog), 6, 2, Qt::AlignLeft | Qt::AlignVCenter);
 
 	QDialogButtonBox* buttons = new QDialogButtonBox(QDialogButtonBox::Close, Qt::Horizontal, &dialog);
-	connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
+	connect(buttons, &QDialogButtonBox::rejected, &dialog, &QDialog::reject);
 	layout->addWidget(buttons, 8, 0, 1, 3);
 
 	dialog.exec();
@@ -285,7 +270,7 @@ void Window::showAbout()
 	QMessageBox::about(this, tr("About Tetzle"), QString("<p align='center'><big><b>%1 %2</b></big><br/>%3<br/><small>%4<br/>%5</small></p>")
 		.arg(tr("Tetzle"), QCoreApplication::applicationVersion(),
 			tr("A jigsaw puzzle with tetrominoes for pieces"),
-			tr("Copyright &copy; 2008-%1 Graeme Gott").arg("2013"),
+			tr("Copyright &copy; 2008-%1 Graeme Gott").arg("2026"),
 			tr("Released under the <a href=%1>GPL 3</a> license").arg("\"http://www.gnu.org/licenses/gpl.html\""))
 	);
 }
